#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Drawing;

namespace HeuristicLab.Core.Networks {
  [Item("PortParameter", "A parameter of a port.")]
  [StorableClass]
  public class PortParameter<T> : NetworkItem, IPortParameter<T> where T : class, IItem {
    public static new Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Field; }
    }
    public override Image ItemImage {
      get {
        if (Type.HasFlag(PortParameterType.Input) && Type.HasFlag(PortParameterType.Output))
          return HeuristicLab.Common.Resources.VSImageLibrary.Field;
        else if (Type.HasFlag(PortParameterType.Input))
          return HeuristicLab.Common.Resources.VSImageLibrary.ArrowDown;
        else if (Type.HasFlag(PortParameterType.Output))
          return HeuristicLab.Common.Resources.VSImageLibrary.ArrowUp;
        else
          return base.ItemImage;
      }
    }

    new public IPort Parent {
      get { return (IPort)base.Parent; }
      set {
        // NOTE: never call setter directly as the Parent property is set by the node which contains the port
        base.Parent = value;
      }
    }
    public Type DataType {
      get { return typeof(T); }
    }
    [Storable]
    private PortParameterType type;
    public PortParameterType Type {
      get { return type; }
      set {
        if (value != type) {
          type = value;
          OnTypeChanged();
        }
      }
    }
    [Storable]
    protected T defaultValue;
    public T DefaultValue {
      get { return defaultValue; }
      set {
        if (value != this.defaultValue) {
          DeregisterDefaultValueEvents();
          this.defaultValue = value;
          RegisterDefaultValueEvents();
          OnDefaultValueChanged();
        }
      }
    }
    IItem IPortParameter.DefaultValue {
      get { return DefaultValue; }
      set {
        T val = value as T;
        if ((value != null) && (val == null))
          throw new InvalidOperationException(
            string.Format("Type mismatch. Value is not a \"{0}\".",
                          typeof(T).GetPrettyName())
          );
        DefaultValue = val;
      }
    }

    [StorableConstructor]
    protected PortParameter(bool deserializing) : base(deserializing) { }
    protected PortParameter(PortParameter<T> original, Cloner cloner)
      : base(original, cloner) {
      this.type = original.type;
      this.defaultValue = cloner.Clone(original.defaultValue);
      RegisterDefaultValueEvents();
    }
    public PortParameter()
      : base("PortParameter") {
      type = PortParameterType.Input;
    }
    public PortParameter(string name)
      : base(name) {
      type = PortParameterType.Input;
    }
    public PortParameter(string name, string description)
      : base(name, description) {
      type = PortParameterType.Input;
    }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterDefaultValueEvents();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PortParameter<T>(this, cloner);
    }

    public override string ToString() {
      var s = base.ToString();
      if (DefaultValue != null)
        s += ": " + DefaultValue.ToString();
      return s;
    }

    public event EventHandler TypeChanged;
    protected virtual void OnTypeChanged() {
      var handler = TypeChanged;
      if (handler != null) handler(this, EventArgs.Empty);
      OnItemImageChanged();
    }

    public event EventHandler DefaultValueChanged;
    protected virtual void OnDefaultValueChanged() {
      var handler = DefaultValueChanged;
      if (handler != null) handler(this, EventArgs.Empty);
      OnToStringChanged();
    }

    public IMessageValue<T> CreateMessageValue() {
      if (DefaultValue == null)
        return new MessageValue<T>(Name);
      else
        return new MessageValue<T>(Name, (T)DefaultValue.Clone());
    }
    IMessageValue IPortParameter.CreateMessageValue() {
      return CreateMessageValue();
    }

    #region DefaultValue Events
    protected virtual void RegisterDefaultValueEvents() {
      if (defaultValue != null) {
        defaultValue.ToStringChanged += new EventHandler(DefaultValue_ToStringChanged);
      }
    }
    protected virtual void DeregisterDefaultValueEvents() {
      if (defaultValue != null) {
        defaultValue.ToStringChanged -= new EventHandler(DefaultValue_ToStringChanged);
      }
    }
    private void DefaultValue_ToStringChanged(object sender, EventArgs e) {
      OnToStringChanged();
    }
    #endregion
  }
}
